"""Copyright © 2022 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

import datetime

def getDateString(date):
    return date.strftime("%d/%m/%Y")

def getDateTimeString(dateTime):
    return dateTime.strftime("%d/%m/%Y %H:%M:%S")

def getDateTimeTimestamp(dateTime):
    return dateTime.strftime("%d.%m.%Y-%H.%M.%S.%f")

def getDateOrDateTimeString(dateOrDateTime):
    if isinstance(dateOrDateTime, datetime.datetime):
        string = getDateTimeString(dateOrDateTime)
    elif isinstance(dateOrDateTime, datetime.date):
        string = getDateString(dateOrDateTime)
    else:
        string = None
    return string

def dateTimeFromString(dateTimeString):
    return datetime.datetime.strptime(dateTimeString, "%d/%m/%Y %H:%M:%S")

def dateFromString(dateString):
    try:
        dateFromString = datetime.datetime.strptime(dateString, "%d/%m/%Y")
    except:
        dateFromString = datetime.datetime.strptime(dateString, "%Y-%m-%d")
    return dateFromString

def dateFromDateTimeOrDateString(dateTimeOrDateString):
    try:
        dateTimeOrDate = dateTimeFromString(dateTimeOrDateString)
    except:
        dateTimeOrDate = dateFromString(dateTimeOrDateString)
    return dateTimeOrDate.date()

def isDateTimeFromDateTimeOrDateString(dateTimeOrDateString):
    try:
        dateTimeOrDate = dateTimeFromString(dateTimeOrDateString)
        isDateTime = True
    except:
        dateTimeOrDate = dateFromString(dateTimeOrDateString)
        isDateTime = False
    return isDateTime

def createDateTimeFromValues(year, month, day, hour, minute, second):
    return datetime.datetime(year, month, day, hour, minute, second)


def roundToSeconds(dateTime):
    microseconds = int(str(dateTime.microsecond)[0])
    if microseconds >= 5:
        secondAdjustment = 1
    else:
        secondAdjustment = 0
    return datetime.datetime(dateTime.year, dateTime.month, dateTime.day, dateTime.hour, dateTime.minute, dateTime.second + secondAdjustment)



"""Copyright © 2022 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

from vectordeprecios.core.Service.Field import Field
from vectordeprecios.util import Enumerations


class FieldQuality(Field.Field):
    def __init__(self, quality=Enumerations.Quality.Quality_Production):
        self.quality = quality

    # quality
    def _get_quality(self):
        return self.__quality
    def _set_quality(self, value):
        if not isinstance(value, Enumerations.Quality):
            raise TypeError("The quality must be set to a class 'Enumerations.Quality'.")
        self.__quality = value
    quality = property(_get_quality, _set_quality)

    # Class Methods

    def getQuality(self):
        return self.quality


"""Copyright © 2022 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

__all__ = ['request']

# Check and install requests libraries if needed
from vectordeprecios.util import Installer, Enumerations, Arguments, Format, Output

Installer.install()

from vectordeprecios.core.Service.ServiceRequest.ServiceRequestDataPoint import ServiceRequestDataPointValue, ServiceRequestDataPointReport
from vectordeprecios.core.Service.ServiceRequest.ServiceRequestDownload import ServiceRequestDownload
import datetime
import sys

def request(token='', ticker='', family='', group='', market='', country=Enumerations.Country.Country_Unspecified,
            document='', item='', chapter='', section='', subsection='', paragraph='', path='',
            dateStart=datetime.datetime.today().date(), dateEnd=datetime.datetime.today().date(),
            fixing=Enumerations.Fixing.EOD, fieldType='', versionType=Enumerations.VersionType.Version_Unspecified,
            author='', version='', quality=Enumerations.Quality.Quality_Unspecified, timeOut=300, timeFrecuency=0.5,
            fileName='', format='dataFrame'):
    if path != '':
        return download(token, path, timeOut, timeFrecuency, format)
    elif not all(reportParameter == '' for reportParameter in [document, item, chapter, section, subsection, paragraph]):
        return requestReport(token, document, item, chapter, section, subsection, paragraph, dateStart, dateEnd, fixing,
                             fieldType, versionType, author, version, quality, timeOut, timeFrecuency, fileName, format)
    else:
        if isinstance(country, str):
            country = Enumerations.Country.fromString(country)
        return requestValue(token, ticker, family, group, market, country, dateStart, dateEnd, fixing, fieldType,
                            versionType, author, version, quality, timeOut, timeFrecuency, fileName, format)

def requestValue(token='', ticker='', family='', group='', market='', country=Enumerations.Country.Country_Unspecified,
                 dateStart=datetime.datetime.today().date(), dateEnd=datetime.datetime.today().date(), fixing=Enumerations.Fixing.EOD,
                 field='', versionType=Enumerations.VersionType.Version_Unspecified, author='',
                 version='', quality=Enumerations.Quality.Quality_Unspecified, timeOut=300, timeFrecuency=0.5, fileName='', format='dataFrame'):
    field = Enumerations.FieldType.fromString(field)
    service = ServiceRequestDataPointValue.ServiceRequestDataPointValue.create(ticker, family, group, market, country, dateStart, dateEnd, fixing, field, versionType, author, version, quality)
    token = Arguments.tokenReader(token)
    url = service.getResponse(token)
    response = download(token, url, timeOut, timeFrecuency, format)
    Output.writeFile(fileName, response)
    return response

def requestReport(token='', document='', item='', chapter='', section='', subsection='', paragraph='',
                  dateStart=datetime.datetime.today().date(), dateEnd=datetime.datetime.today().date(), fixing=Enumerations.Fixing.EOD,
                  field='', versionType=Enumerations.VersionType.Version_Unspecified, author='',
                  version='', quality=Enumerations.Quality.Quality_Unspecified, timeOut=300, timeFrecuency=0.5, fileName='', format='dataFrame'):
    field = Enumerations.FieldType.fromString(field)
    service = ServiceRequestDataPointReport.ServiceRequestDataPointReport.create(document, item, chapter, section, subsection, paragraph, dateStart, dateEnd, fixing, field, versionType, author, version, quality)
    token = Arguments.tokenReader(token)
    url = service.getResponse(token)
    response = download(token, url, timeOut, timeFrecuency, format)
    Output.writeFile(fileName, response)
    return response

def download(token='', path='', timeOut=300, timeFrecuency=0.5, format='dataFrame'):
    response = ServiceRequestDownload.ServiceRequestDownload.request(token, path, timeOut, timeFrecuency)
    try:
        format = Enumerations.Format.fromString(format)
        response = Format.jsonToFormat(format, response)
    except:
        print('Response is not values or reports, therefore it will be returned raw.')
    return response

if __name__=='__main__':
    if not len(sys.argv) > 1:
        token = Arguments.tokenReader()
        # Request Banco de Chile stock price
        response = request(token, ticker='CHILE')
    else:
        # For command line execution
        args = Arguments.parseArguments()
        country, dateStart, dateEnd, fixing, fieldType, versionType, quality = Arguments.checkArguments(args)
        # Service
        if args.service == 'r':
            request = request(args.token, args.ticker, args.family, args.group, args.market, country, dateStart, dateEnd, fixing, fieldType, versionType, args.author, args.version, quality, args.timeOut, args.timeFrecuency, args.filepath)



"""Copyright © 2022 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

import json, datetime
from vectordeprecios.util import Enumerations


class Field:
    @staticmethod
    def __serializeAndClean(dictionary):
        delete = []
        # Serialize complex objects
        for key, value in zip(dictionary.keys(), dictionary.values()):
            if value in ['', Enumerations.Country.Country_Unspecified]:
                delete.append(key)
            elif isinstance(value, Enumerations.Enum):
                dictionary[key] = value.value
            elif isinstance(value, (datetime.date, datetime.datetime)):
                dictionary[key] = value.isoformat()
        # Delete empty attributes
        for key in delete:
            del dictionary[key]
        return dictionary

    def getJson(self, pythonDict=False):
        # Get all attributes as a dictionary
        data = self.__dict__.copy()
        # Delete object private prefix from attribute names
        oldKeys = list(data.keys())
        for key in oldKeys:
            data[key.replace('_' + self.__class__.__name__ + '__', '')] = data.pop(key)
        # Serialize and clean dictionary
        data = self.__serializeAndClean(data)
        # jsonize data or keep as a Python dictionary
        if pythonDict:
            return data
        else:
            return json.dumps(data)
